OSDN Git Service

iomap: Switch from blkno to disk offset
authorAndreas Gruenbacher <agruenba@redhat.com>
Sun, 1 Oct 2017 21:55:54 +0000 (17:55 -0400)
committerTheodore Ts'o <tytso@mit.edu>
Sun, 1 Oct 2017 21:55:54 +0000 (17:55 -0400)
Replace iomap->blkno, the sector number, with iomap->addr, the disk
offset in bytes.  For invalid disk offsets, use the special value
IOMAP_NULL_ADDR instead of IOMAP_NULL_BLOCK.

This allows to use iomap for mappings which are not block aligned, such
as inline data on ext4.

Signed-off-by: Andreas Gruenbacher <agruenba@redhat.com>
Signed-off-by: Theodore Ts'o <tytso@mit.edu>
Reviewed-by: Darrick J. Wong <darrick.wong@oracle.com> # iomap, xfs
Reviewed-by: Jan Kara <jack@suse.cz>
fs/buffer.c
fs/dax.c
fs/ext2/inode.c
fs/ext4/inode.c
fs/iomap.c
fs/nfsd/blocklayout.c
fs/xfs/xfs_iomap.c
include/linux/iomap.h

index 170df85..bd4d092 100644 (file)
@@ -1978,8 +1978,8 @@ iomap_to_bh(struct inode *inode, sector_t block, struct buffer_head *bh,
        case IOMAP_MAPPED:
                if (offset >= i_size_read(inode))
                        set_buffer_new(bh);
-               bh->b_blocknr = (iomap->blkno >> (inode->i_blkbits - 9)) +
-                               ((offset - iomap->offset) >> inode->i_blkbits);
+               bh->b_blocknr = (iomap->addr + offset - iomap->offset) >>
+                               inode->i_blkbits;
                set_buffer_mapped(bh);
                break;
        }
index f001d8c..f3a44a7 100644 (file)
--- a/fs/dax.c
+++ b/fs/dax.c
@@ -938,7 +938,7 @@ EXPORT_SYMBOL_GPL(__dax_zero_page_range);
 
 static sector_t dax_iomap_sector(struct iomap *iomap, loff_t pos)
 {
-       return iomap->blkno + (((pos & PAGE_MASK) - iomap->offset) >> 9);
+       return (iomap->addr + (pos & PAGE_MASK) - iomap->offset) >> 9;
 }
 
 static loff_t
index 4dca6f3..1b8fc73 100644 (file)
@@ -820,11 +820,11 @@ static int ext2_iomap_begin(struct inode *inode, loff_t offset, loff_t length,
 
        if (ret == 0) {
                iomap->type = IOMAP_HOLE;
-               iomap->blkno = IOMAP_NULL_BLOCK;
+               iomap->addr = IOMAP_NULL_ADDR;
                iomap->length = 1 << blkbits;
        } else {
                iomap->type = IOMAP_MAPPED;
-               iomap->blkno = (sector_t)bno << (blkbits - 9);
+               iomap->addr = (u64)bno << blkbits;
                iomap->length = (u64)ret << blkbits;
                iomap->flags |= IOMAP_F_MERGED;
        }
index 31db875..d9e633c 100644 (file)
@@ -3472,7 +3472,7 @@ retry:
 
        if (ret == 0) {
                iomap->type = IOMAP_HOLE;
-               iomap->blkno = IOMAP_NULL_BLOCK;
+               iomap->addr = IOMAP_NULL_ADDR;
                iomap->length = (u64)map.m_len << blkbits;
        } else {
                if (map.m_flags & EXT4_MAP_MAPPED) {
@@ -3483,7 +3483,7 @@ retry:
                        WARN_ON_ONCE(1);
                        return -EIO;
                }
-               iomap->blkno = (sector_t)map.m_pblk << (blkbits - 9);
+               iomap->addr = (u64)map.m_pblk << blkbits;
                iomap->length = (u64)map.m_len << blkbits;
        }
 
index be61cf7..d25db03 100644 (file)
@@ -350,8 +350,8 @@ static int iomap_zero(struct inode *inode, loff_t pos, unsigned offset,
 static int iomap_dax_zero(loff_t pos, unsigned offset, unsigned bytes,
                struct iomap *iomap)
 {
-       sector_t sector = iomap->blkno +
-               (((pos & ~(PAGE_SIZE - 1)) - iomap->offset) >> 9);
+       sector_t sector = (iomap->addr +
+                          (pos & PAGE_MASK) - iomap->offset) >> 9;
 
        return __dax_zero_page_range(iomap->bdev, iomap->dax_dev, sector,
                        offset, bytes);
@@ -512,9 +512,8 @@ static int iomap_to_fiemap(struct fiemap_extent_info *fi,
                flags |= FIEMAP_EXTENT_SHARED;
 
        return fiemap_fill_next_extent(fi, iomap->offset,
-                       iomap->blkno != IOMAP_NULL_BLOCK ? iomap->blkno << 9: 0,
+                       iomap->addr != IOMAP_NULL_ADDR ? iomap->addr : 0,
                        iomap->length, flags);
-
 }
 
 static loff_t
@@ -823,7 +822,7 @@ iomap_dio_zero(struct iomap_dio *dio, struct iomap *iomap, loff_t pos,
        bio = bio_alloc(GFP_KERNEL, 1);
        bio_set_dev(bio, iomap->bdev);
        bio->bi_iter.bi_sector =
-               iomap->blkno + ((pos - iomap->offset) >> 9);
+               (iomap->addr + pos - iomap->offset) >> 9;
        bio->bi_private = dio;
        bio->bi_end_io = iomap_dio_bio_end_io;
 
@@ -902,7 +901,7 @@ iomap_dio_actor(struct inode *inode, loff_t pos, loff_t length,
                bio = bio_alloc(GFP_KERNEL, nr_pages);
                bio_set_dev(bio, iomap->bdev);
                bio->bi_iter.bi_sector =
-                       iomap->blkno + ((pos - iomap->offset) >> 9);
+                       (iomap->addr + pos - iomap->offset) >> 9;
                bio->bi_write_hint = dio->iocb->ki_hint;
                bio->bi_private = dio;
                bio->bi_end_io = iomap_dio_bio_end_io;
index c862c24..2d1d37b 100644 (file)
@@ -65,7 +65,7 @@ nfsd4_block_proc_layoutget(struct inode *inode, const struct svc_fh *fhp,
                        bex->es = PNFS_BLOCK_READ_DATA;
                else
                        bex->es = PNFS_BLOCK_READWRITE_DATA;
-               bex->soff = (iomap.blkno << 9);
+               bex->soff = iomap.addr;
                break;
        case IOMAP_UNWRITTEN:
                if (seg->iomode & IOMODE_RW) {
@@ -78,7 +78,7 @@ nfsd4_block_proc_layoutget(struct inode *inode, const struct svc_fh *fhp,
                        }
 
                        bex->es = PNFS_BLOCK_INVALID_DATA;
-                       bex->soff = (iomap.blkno << 9);
+                       bex->soff = iomap.addr;
                        break;
                }
                /*FALLTHRU*/
index f179bdf..9744b48 100644 (file)
@@ -54,13 +54,13 @@ xfs_bmbt_to_iomap(
        struct xfs_mount        *mp = ip->i_mount;
 
        if (imap->br_startblock == HOLESTARTBLOCK) {
-               iomap->blkno = IOMAP_NULL_BLOCK;
+               iomap->addr = IOMAP_NULL_ADDR;
                iomap->type = IOMAP_HOLE;
        } else if (imap->br_startblock == DELAYSTARTBLOCK) {
-               iomap->blkno = IOMAP_NULL_BLOCK;
+               iomap->addr = IOMAP_NULL_ADDR;
                iomap->type = IOMAP_DELALLOC;
        } else {
-               iomap->blkno = xfs_fsb_to_db(ip, imap->br_startblock);
+               iomap->addr = BBTOB(xfs_fsb_to_db(ip, imap->br_startblock));
                if (imap->br_state == XFS_EXT_UNWRITTEN)
                        iomap->type = IOMAP_UNWRITTEN;
                else
index f64dc6c..7b8a615 100644 (file)
@@ -15,8 +15,8 @@ struct vm_fault;
  */
 #define IOMAP_HOLE     0x01    /* no blocks allocated, need allocation */
 #define IOMAP_DELALLOC 0x02    /* delayed allocation blocks */
-#define IOMAP_MAPPED   0x03    /* blocks allocated @blkno */
-#define IOMAP_UNWRITTEN        0x04    /* blocks allocated @blkno in unwritten state */
+#define IOMAP_MAPPED   0x03    /* blocks allocated at @addr */
+#define IOMAP_UNWRITTEN        0x04    /* blocks allocated at @addr in unwritten state */
 
 /*
  * Flags for all iomap mappings:
@@ -30,12 +30,12 @@ struct vm_fault;
 #define IOMAP_F_SHARED 0x20    /* block shared with another file */
 
 /*
* Magic value for blkno:
  */
-#define IOMAP_NULL_BLOCK -1LL  /* blkno is not valid */
+#define IOMAP_NULL_ADDR -1ULL  /* addr is not valid */
 
 struct iomap {
-       sector_t                blkno;  /* 1st sector of mapping, 512b units */
+       u64                     addr; /* disk offset of mapping, bytes */
        loff_t                  offset; /* file offset of mapping, bytes */
        u64                     length; /* length of mapping, bytes */
        u16                     type;   /* type of mapping */
@@ -30,12 +30,12 @@ struct vm_fault;
 
 /*
- * Magic value for blkno:
+ * Magic value for addr:
  */